from pathlib import Path

from ..writer import (
    BaseDataWriter,
    PyProjection,
    PyProjectionGeometry,
    PyRegionOfIntrest,
    PyVolume,
)

try:
    from PythonTools.py2raw import py2raw
    from PythonTools.ezrt_header import EzrtHeader
    from PythonTools.py2rek import py2rek
except ModuleNotFoundError:
    raise ModuleNotFoundError("Install PythonTools from Fraunhofer EZRT.")


from ..rq_json.json_write import RqJsonWriter


class RawWriter(BaseDataWriter):
    def __init__(self):
        super().__init__(".raw", ".raw", ".roi-json", ".rek")

    def write_projection_geometry(
        self, save_path: Path, projection_geometry: PyProjectionGeometry
    ):
        header = EzrtHeader()
        header.agv_source_position = (
            projection_geometry.focal_spot_mm / 1000.0
        ).tolist()
        header.agv_detector_center_position = (
            projection_geometry.detector_postion_mm / 1000.0
        ).tolist()

        line = projection_geometry.detector_rotation_matrix[:, 0]
        column = projection_geometry.detector_rotation_matrix[:, 0]

        header.agv_detector_line_direction = line
        header.agv_detector_col_direction = column

        header.number_of_images = projection_geometry.frame_id
        header.tofile(save_path)

    def write_projection(
        self, save_path: Path, projection: PyProjection, swith_order: bool = False
    ):
        projection_geometry = projection.get_projection_geometry()

        header = EzrtHeader()

        header.agv_source_position = (
            projection_geometry.focal_spot_mm / 1000.0
        ).tolist()
        header.agv_detector_center_position = (
            projection_geometry.detector_postion_mm / 1000.0
        ).tolist()

        line = projection_geometry.detector_rotation_matrix[:, 0]
        column = projection_geometry.detector_rotation_matrix[:, 0]

        header.agv_detector_line_direction = line
        header.agv_detector_col_direction = column

        header.number_of_images = projection_geometry.frame_id
        header.pixel_width_in_um = projection.pixel_pitch_x_mm * 1000.0
        header.voltage_in_kv = projection.voltage_kv
        header.current_in_ua = projection.current_ua
        header.exposure_time_in_ms = projection.exposure_time_ms

        py2raw(projection.image, save_path, header, swith_order)

    def write_region_of_intrest(
        self, save_path: Path, region_of_intrest: PyRegionOfIntrest
    ):
        RqJsonWriter.write_region_of_intrest(self, save_path, region_of_intrest)

    def write_volume(self, save_path: Path, volume: PyVolume):
        header = EzrtHeader()
        header.num_voxel_x = volume.shape[0]
        header.num_voxel_y = volume.shape[1]
        header.num_voxel_z = volume.shape[2]
        header.pixel_width_in_um = volume.roi.resolution_mm[0]

        py2rek(volume.array, save_path, header, True)
